@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -145,7 +145,8 @@ public abstract class ClasspathFontPeer
             uname.equals ("SERIF") ||
             uname.equals ("MONOSPACED") ||
             uname.equals ("DIALOG") ||
-            uname.equals ("DIALOGINPUT"));
+            uname.equals ("DIALOGINPUT") ||
+            uname.equals ("DEFAULT"));
   }
 
   protected static String logicalFontNameToFaceName (String name)
@@ -161,6 +162,8 @@ public abstract class ClasspathFontPeer
       return "Helvetica";
     else if (uname.equals ("DIALOGINPUT"))
       return "Helvetica";
+    else if (uname.equals ("DEFAULT"))
+      return "Dialog.plain";
     else
       return "Helvetica";
   }
@@ -233,7 +236,7 @@ public abstract class ClasspathFontPeer
       family = (String) attribs.get (TextAttribute.FAMILY);
 
     if (name == null)
-      name = "SansSerif";
+      name = "Default";
 
     if (attribs.containsKey (TextAttribute.WEIGHT))
       {
@@ -829,18 +832,4 @@ public abstract class ClasspathFontPeer
   public abstract Rectangle2D getMaxCharBounds (Font font, 
                                                 FontRenderContext rc);
 
-  /** 
-   * Implementation of {@link Font#getStringBounds(CharacterIterator, int,
-   * int, FontRenderContext)}
-   *
-   * @param font the font this peer is being called from. This may be
-   * useful if you are sharing peers between Font objects. Otherwise it may
-   * be ignored.
-   */
-
-  public abstract Rectangle2D getStringBounds (Font font, 
-                                               CharacterIterator ci, 
-                                               int begin, int limit, 
-                                               FontRenderContext frc);
-
 }
