@@ -1,5 +1,5 @@
 /* GtkListPeer.java -- Implements ListPeer with GTK
-   Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2006 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -40,9 +40,10 @@ package gnu.java.awt.peer.gtk;
 
 import java.awt.AWTEvent;
 import java.awt.Dimension;
+import java.awt.FontMetrics;
 import java.awt.List;
-import java.awt.event.MouseEvent;
 import java.awt.event.KeyEvent;
+import java.awt.event.MouseEvent;
 import java.awt.peer.ListPeer;
 
 public class GtkListPeer extends GtkComponentPeer
@@ -58,9 +59,13 @@ public class GtkListPeer extends GtkComponentPeer
   }
 
   native void create (int rows);
-  native void connectJObject ();
   native void connectSignals ();
-  native void gtkSetFont (String name, int style, int size);
+
+  /**
+   * Overridden to set the Font of the text insode the gtk_scrolled_window.
+   */
+  protected native void gtkWidgetModifyFont (String name, int style, int size);
+
   native void gtkWidgetRequestFocus ();
 
   native void getSize (int rows, int visibleRows, int dims[]);
@@ -116,11 +121,12 @@ public class GtkListPeer extends GtkComponentPeer
 
   public Dimension preferredSize (int rows)
   {
-    int dims[] = new int[2];
-
-    int visibleRows = ((List) awtComponent).getRows();
-    getSize (rows, visibleRows, dims);
-    return new Dimension (dims[0], dims[1]);
+    // getSize returns the minimum size of the list.
+    // The width is too narrow for a typical list.
+    List l = (List) awtComponent;
+    Dimension d = getMinimumSize();
+    FontMetrics fm = l.getFontMetrics(l.getFont());
+    return new Dimension(d.width + fm.stringWidth("1234567890"), d.height);
   }
 
   public void removeAll ()
