@@ -1,5 +1,5 @@
 /* Request.java --
-   Copyright (C) 2004 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2006 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -40,7 +40,6 @@ package gnu.java.net.protocol.http;
 
 import gnu.java.net.BASE64;
 import gnu.java.net.LineInputStream;
-import gnu.java.net.protocol.http.event.RequestEvent;
 
 import java.io.IOException;
 import java.io.InputStream;
@@ -94,16 +93,6 @@ public class Request
    */
   protected RequestBodyWriter requestBodyWriter;
 
-  /**
-   * Request body negotiation threshold for 100-continue expectations.
-   */
-  protected int requestBodyNegotiationThreshold;
-
-  /**
-   * The response body reader.
-   */
-  protected ResponseBodyReader responseBodyReader;
-
   /**
    * Map of response header handlers.
    */
@@ -133,7 +122,6 @@ public class Request
     this.path = path;
     requestHeaders = new Headers();
     responseHeaderHandlers = new HashMap();
-    requestBodyNegotiationThreshold = 4096;
   }
 
   /**
@@ -235,16 +223,6 @@ public class Request
     this.requestBodyWriter = requestBodyWriter;
   }
 
-  /**
-   * Sets the response body reader.
-   * @param responseBodyReader the handler to receive notifications of
-   * response body content
-   */
-  public void setResponseBodyReader(ResponseBodyReader responseBodyReader)
-  {
-    this.responseBodyReader = responseBodyReader;
-  }
-
   /**
    * Sets a callback handler to be invoked for the specified header name.
    * @param name the header name
@@ -266,21 +244,6 @@ public class Request
     this.authenticator = authenticator;
   }
 
-  /**
-   * Sets the request body negotiation threshold.
-   * If this is set, it determines the maximum size that the request body
-   * may be before body negotiation occurs(via the
-   * <code>100-continue</code> expectation). This ensures that a large
-   * request body is not sent when the server wouldn't have accepted it
-   * anyway.
-   * @param threshold the body negotiation threshold, or &lt;=0 to disable
-   * request body negotation entirely
-   */
-  public void setRequestBodyNegotiationThreshold(int threshold)
-  {
-    requestBodyNegotiationThreshold = threshold;
-  }
-
   /**
    * Dispatches this request.
    * A request can only be dispatched once; calling this method a second
@@ -307,10 +270,10 @@ public class Request
     if (requestBodyWriter != null)
       {
         contentLength = requestBodyWriter.getContentLength();
-        if (contentLength > requestBodyNegotiationThreshold)
+        String expect = getHeader("Expect");
+        if (expect != null && expect.equals("100-continue"))
           {
             expectingContinue = true;
-            setHeader("Expect", "100-continue");
           }
         else
           {
@@ -324,13 +287,10 @@ public class Request
         do
           {
             retry = false;
-            // Send request
-            connection.fireRequestEvent(RequestEvent.REQUEST_SENDING, this);
             
             // Get socket output and input streams
             OutputStream out = connection.getOutputStream();
-            LineInputStream in =
-              new LineInputStream(connection.getInputStream());
+
             // Request line
             String requestUri = path;
             if (connection.isUsingProxy() &&
@@ -342,12 +302,10 @@ public class Request
             String line = method + ' ' + requestUri + ' ' + version + CRLF;
             out.write(line.getBytes(US_ASCII));
             // Request headers
-            for (Iterator i = requestHeaders.keySet().iterator();
-                 i.hasNext(); )
+            for (Iterator i = requestHeaders.iterator(); i.hasNext(); )
               {
-                String name =(String) i.next();
-                String value =(String) requestHeaders.get(name);
-                line = name + HEADER_SEP + value + CRLF;
+                Headers.HeaderElement elt = (Headers.HeaderElement)i.next();
+                line = elt.name + HEADER_SEP + elt.value + CRLF;
                 out.write(line.getBytes(US_ASCII));
               }
             out.write(CRLF.getBytes(US_ASCII));
@@ -369,28 +327,42 @@ public class Request
                     count += len;
                   }
                 while (len > -1 && count < contentLength);
-                out.write(CRLF.getBytes(US_ASCII));
               }
             out.flush();
-            // Sent event
-            connection.fireRequestEvent(RequestEvent.REQUEST_SENT, this);
             // Get response
-            response = readResponse(in);
-            int sc = response.getCode();
-            if (sc == 401 && authenticator != null)
-              {
-                if (authenticate(response, attempts++))
-                  {
-                    retry = true;
-                  }
-              }
-            else if (sc == 100 && expectingContinue)
-              {
-                requestHeaders.remove("Expect");
-                setHeader("Content-Length", Integer.toString(contentLength));
-                expectingContinue = false;
-                retry = true;
-              }
+            while(true)
+            {
+              response = readResponse(connection.getInputStream());
+              int sc = response.getCode();
+              if (sc == 401 && authenticator != null)
+                {
+                  if (authenticate(response, attempts++))
+                    {
+                      retry = true;
+                    }
+                }
+              else if (sc == 100)
+                {
+                  if (expectingContinue)
+                    {
+                      requestHeaders.remove("Expect");
+                      setHeader("Content-Length",
+                                Integer.toString(contentLength));
+                      expectingContinue = false;
+                      retry = true;
+                    }
+                  else
+                    {
+                      // A conforming server can send an unsoliceted
+                      // Continue response but *should* not (RFC 2616
+                      // sec 8.2.3).  Ignore the bogus Continue
+                      // response and get the real response that
+                      // should follow
+                      continue;
+                    }
+                }
+              break;
+            }
           }
         while (retry);
       }
@@ -402,14 +374,16 @@ public class Request
     return response;
   }
     
-  Response readResponse(LineInputStream in)
+  Response readResponse(InputStream in)
     throws IOException
   {
     String line;
     int len;
     
     // Read response status line
-    line = in.readLine();
+    LineInputStream lis = new LineInputStream(in);
+
+    line = lis.readLine();
     if (line == null)
       {
         throw new ProtocolException("Peer closed connection");
@@ -438,74 +412,87 @@ public class Request
     String message = line.substring(end + 1, len - 1);
     // Read response headers
     Headers responseHeaders = new Headers();
-    responseHeaders.parse(in);
+    responseHeaders.parse(lis);
     notifyHeaderHandlers(responseHeaders);
-    // Construct response
-    int codeClass = code / 100;
-    Response ret = new Response(majorVersion, minorVersion, code,
-                                codeClass, message, responseHeaders);
+    InputStream body = null;
+    
     switch (code)
       {
+      case 100:
+        break;
       case 204:
       case 205:
       case 304:
+        body = createResponseBodyStream(responseHeaders, majorVersion,
+                                        minorVersion, in, false);
         break;
       default:
-        // Does response body reader want body?
-        boolean notify = (responseBodyReader != null);
-        if (notify)
-          {
-            if (!responseBodyReader.accept(this, ret))
-              {
-                notify = false;
-              }
-          }
-        readResponseBody(ret, in, notify);
+        body = createResponseBodyStream(responseHeaders, majorVersion,
+                                        minorVersion, in, true);
       }
+
+    // Construct response
+    Response ret = new Response(majorVersion, minorVersion, code,
+                                message, responseHeaders, body);
     return ret;
   }
 
   void notifyHeaderHandlers(Headers headers)
   {
-    for (Iterator i = headers.entrySet().iterator(); i.hasNext(); )
+    for (Iterator i = headers.iterator(); i.hasNext(); )
       {
-        Map.Entry entry = (Map.Entry) i.next();
-        String name =(String) entry.getKey();
+        Headers.HeaderElement entry = (Headers.HeaderElement) i.next();
         // Handle Set-Cookie
-        if ("Set-Cookie".equalsIgnoreCase(name))
-          {
-            String value = (String) entry.getValue();
-            handleSetCookie(value);
-          }
+        if ("Set-Cookie".equalsIgnoreCase(entry.name))
+            handleSetCookie(entry.value);
+
         ResponseHeaderHandler handler =
-          (ResponseHeaderHandler) responseHeaderHandlers.get(name);
+          (ResponseHeaderHandler) responseHeaderHandlers.get(entry.name);
         if (handler != null)
-          {
-            String value = (String) entry.getValue();
-            handler.setValue(value);
-          }
+            handler.setValue(entry.value);
       }
   }
 
-  void readResponseBody(Response response, InputStream in,
-                        boolean notify)
+  private InputStream createResponseBodyStream(Headers responseHeaders,
+                                               int majorVersion,
+                                               int minorVersion,
+                                               InputStream in,
+                                               boolean mayHaveBody)
     throws IOException
   {
-    byte[] buffer = new byte[4096];
-    int contentLength = -1;
+    long contentLength = -1;
     Headers trailer = null;
     
-    String transferCoding = response.getHeader("Transfer-Encoding");
-    if ("chunked".equalsIgnoreCase(transferCoding))
+    // Persistent connections are the default in HTTP/1.1
+    boolean doClose = "close".equalsIgnoreCase(getHeader("Connection")) ||
+      "close".equalsIgnoreCase(responseHeaders.getValue("Connection")) ||
+      (connection.majorVersion == 1 && connection.minorVersion == 0) ||
+      (majorVersion == 1 && minorVersion == 0);
+
+    String transferCoding = responseHeaders.getValue("Transfer-Encoding");
+    if ("HEAD".equals(method) || !mayHaveBody)
       {
-        trailer = new Headers();
-        in = new ChunkedInputStream(in, trailer);
+        // Special case no body.
+        in = new LimitedLengthInputStream(in, 0, true, connection, doClose);
+      }
+    else if ("chunked".equalsIgnoreCase(transferCoding))
+      {
+        in = new LimitedLengthInputStream(in, -1, false, connection, doClose);
+          
+        in = new ChunkedInputStream(in, responseHeaders);
       } 
     else
       {
-        contentLength = response.getIntHeader("Content-Length");
+        contentLength = responseHeaders.getLongValue("Content-Length");
+
+        if (contentLength < 0)
+          doClose = true;  // No Content-Length, must close.
+
+        in = new LimitedLengthInputStream(in, contentLength,
+                                          contentLength >= 0,
+                                          connection, doClose);
       }
-    String contentCoding = response.getHeader("Content-Encoding");
+    String contentCoding = responseHeaders.getValue("Content-Encoding");
     if (contentCoding != null && !"identity".equals(contentCoding))
       {
         if ("gzip".equals(contentCoding))
@@ -521,52 +508,11 @@ public class Request
             throw new ProtocolException("Unsupported Content-Encoding: " +
                                         contentCoding);
           }
+	// Remove the Content-Encoding header because the content is
+	// no longer compressed.
+	responseHeaders.remove("Content-Encoding");
       }
-    
-    // Persistent connections are the default in HTTP/1.1
-    boolean doClose = "close".equalsIgnoreCase(getHeader("Connection")) ||
-      "close".equalsIgnoreCase(response.getHeader("Connection")) ||
-      (connection.majorVersion == 1 && connection.minorVersion == 0) ||
-      (response.majorVersion == 1 && response.minorVersion == 0);
-    
-    int count = contentLength;
-    int len = (count > -1) ? count : buffer.length;
-    len = (len > buffer.length) ? buffer.length : len;
-    while (len > -1)
-      {
-        len = in.read(buffer, 0, len);
-        if (len < 0)
-          {
-            // EOF
-            connection.closeConnection();
-            break;
-          }
-        if (notify)
-          {
-            responseBodyReader.read(buffer, 0, len);
-          }
-        if (count > -1)
-          {
-            count -= len;
-            if (count < 1)
-              {
-                if (doClose)
-                  {
-                    connection.closeConnection();
-                  }
-                break;
-              }
-          }
-      }
-    if (notify)
-      {
-        responseBodyReader.close();
-      }
-    if (trailer != null)
-      {
-        response.getHeaders().putAll(trailer);
-        notifyHeaderHandlers(trailer);
-      }
+    return in;
   }
 
   boolean authenticate(Response response, int attempts)
@@ -686,7 +632,7 @@ public class Request
   {
     int len = text.length();
     String key = null;
-    StringBuffer buf = new StringBuffer();
+    StringBuilder buf = new StringBuilder();
     Properties ret = new Properties();
     boolean inQuote = false;
     for (int i = 0; i < len; i++)
@@ -739,7 +685,7 @@ public class Request
   {
     int nc = connection.getNonceCount(nonce);
     String hex = Integer.toHexString(nc);
-    StringBuffer buf = new StringBuffer();
+    StringBuilder buf = new StringBuilder();
     for (int i = 8 - hex.length(); i > 0; i--)
       {
         buf.append('0');
@@ -810,7 +756,7 @@ public class Request
 
     int len = text.length();
     String attr = null;
-    StringBuffer buf = new StringBuffer();
+    StringBuilder buf = new StringBuilder();
     boolean inQuote = false;
     for (int i = 0; i <= len; i++)
       {
