@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -37,17 +37,29 @@ exception statement from your version. */
 
 package gnu.xml.pipeline;
 
-import java.util.Hashtable;
+import gnu.xml.util.DomParser;
 
-import org.w3c.dom.*;
-import org.xml.sax.*;
+import org.xml.sax.Attributes;
+import org.xml.sax.ContentHandler;
+import org.xml.sax.DTDHandler;
+import org.xml.sax.ErrorHandler;
+import org.xml.sax.Locator;
+import org.xml.sax.SAXException;
+import org.xml.sax.SAXNotRecognizedException;
+import org.xml.sax.SAXParseException;
 import org.xml.sax.ext.DeclHandler;
 import org.xml.sax.ext.LexicalHandler;
 import org.xml.sax.helpers.AttributesImpl;
-
-import gnu.xml.aelfred2.ContentHandler2;
-import gnu.xml.util.DomParser;
-
+import org.w3c.dom.Attr;
+import org.w3c.dom.CDATASection;
+import org.w3c.dom.CharacterData;
+import org.w3c.dom.Document;
+import org.w3c.dom.DOMImplementation;
+import org.w3c.dom.Element;
+import org.w3c.dom.EntityReference;
+import org.w3c.dom.Node;
+import org.w3c.dom.ProcessingInstruction;
+import org.w3c.dom.Text;
 
 /**
  * This consumer builds a DOM Document from its input, acting either as a
@@ -317,7 +329,7 @@ public class DomConsumer implements EventConsumer
      * accepted illegal input data). </p>
      */
     public static class Handler
-	implements ContentHandler2, LexicalHandler,
+	implements ContentHandler, LexicalHandler,
 	    DTDHandler, DeclHandler
     {
 	protected DomConsumer		consumer;
@@ -417,20 +429,6 @@ public class DomConsumer implements EventConsumer
 	    top = document;
 	}
 
-        // ContentHandler2
-        public void xmlDecl(String version,
-                            String encoding,
-                            boolean standalone,
-                            String inputEncoding)
-          throws SAXException
-        {
-          if (document != null)
-            {
-              document.setXmlVersion(version);
-              document.setXmlStandalone(standalone);
-            }
-        }
-
 	// SAX1
 	public void endDocument ()
 	throws SAXException
