@@ -39,6 +39,7 @@ exception statement from your version. */
 package java.awt;
 
 import java.applet.Applet;
+import java.awt.FocusTraversalPolicy;
 import java.awt.event.FocusEvent;
 import java.awt.event.KeyEvent;
 import java.awt.event.WindowEvent;
@@ -213,7 +214,7 @@ public abstract class KeyboardFocusManager
       currentFocusOwners */
   private static Map currentFocusCycleRoots = new HashMap ();
 
-  /** The default {@link FocusTraveralPolicy} that focus-managing
+  /** The default {@link FocusTraversalPolicy} that focus-managing
       {@link Container}s will use to define their initial focus
       traversal policy. */
   private FocusTraversalPolicy defaultPolicy;
@@ -287,53 +288,13 @@ public abstract class KeyboardFocusManager
     KeyboardFocusManager manager;
 
     if (m == null)
-      manager = createFocusManager();
+      manager = new DefaultKeyboardFocusManager();
     else
       manager = m;
 
     currentKeyboardFocusManagers.put (currentGroup, manager);
   }
 
-  /**
-   * Creates a KeyboardFocusManager. The exact class is determined by the
-   * system property 'gnu.java.awt.FocusManager'. If this is not set,
-   * we default to DefaultKeyboardFocusManager.
-   */
-  private static KeyboardFocusManager createFocusManager()
-  {
-    String fmClassName = System.getProperty("gnu.java.awt.FocusManager",
-                                       "java.awt.DefaultKeyboardFocusManager");
-    try
-      {
-        Class fmClass = Class.forName(fmClassName);
-        KeyboardFocusManager fm = (KeyboardFocusManager) fmClass.newInstance();
-        return fm;
-      }
-    catch (ClassNotFoundException ex)
-      {
-        System.err.println("The class " + fmClassName + " cannot be found.");
-        System.err.println("Check the setting of the system property");
-        System.err.println("gnu.java.awt.FocusManager");
-        return null;
-      }
-    catch (InstantiationException ex)
-      {
-        System.err.println("The class " + fmClassName + " cannot be");
-        System.err.println("instantiated.");
-        System.err.println("Check the setting of the system property");
-        System.err.println("gnu.java.awt.FocusManager");
-        return null;
-      }
-    catch (IllegalAccessException ex)
-      {
-        System.err.println("The class " + fmClassName + " cannot be");
-        System.err.println("accessed.");
-        System.err.println("Check the setting of the system property");
-        System.err.println("gnu.java.awt.FocusManager");
-        return null;
-      }
-  }
-
   /**
    * Retrieve the {@link Component} that has the keyboard focus, or
    * null if the focus owner was not set by a thread in the current
@@ -343,10 +304,7 @@ public abstract class KeyboardFocusManager
    */
   public Component getFocusOwner ()
   {
-    Component owner = (Component) getObject (currentFocusOwners);
-    if (owner == null)
-      owner = (Component) getObject (currentPermanentFocusOwners);
-    return owner;
+    return (Component) getObject (currentFocusOwners);
   }
 
   /**
@@ -362,10 +320,7 @@ public abstract class KeyboardFocusManager
    */
   protected Component getGlobalFocusOwner ()
   {
-    // Check if there is a temporary focus owner.
-    Component focusOwner = (Component) getGlobalObject (currentFocusOwners);
-
-    return (focusOwner == null) ? getGlobalPermanentFocusOwner () : focusOwner;
+    return (Component) getGlobalObject(currentFocusOwners, true);
   }
 
   /**
@@ -448,7 +403,7 @@ public abstract class KeyboardFocusManager
    */
   protected Component getGlobalPermanentFocusOwner ()
   {
-    return (Component) getGlobalObject (currentPermanentFocusOwners);
+    return (Component) getGlobalObject (currentPermanentFocusOwners, true);
   }
 
   /**
@@ -494,7 +449,7 @@ public abstract class KeyboardFocusManager
    */
   protected Window getGlobalFocusedWindow ()
   {
-    return (Window) getGlobalObject (currentFocusedWindows);
+    return (Window) getGlobalObject (currentFocusedWindows, true);
   }
 
   /**
@@ -536,7 +491,7 @@ public abstract class KeyboardFocusManager
    */
   protected Window getGlobalActiveWindow()
   {
-    return (Window) getGlobalObject (currentActiveWindows);
+    return (Window) getGlobalObject (currentActiveWindows, true);
   }
 
   /**
@@ -702,7 +657,7 @@ public abstract class KeyboardFocusManager
    */
   protected Container getGlobalCurrentFocusCycleRoot ()
   {
-    return (Container) getGlobalObject (currentFocusCycleRoots);
+    return (Container) getGlobalObject (currentFocusCycleRoots, true);
   }
 
   /**
@@ -1144,11 +1099,9 @@ public abstract class KeyboardFocusManager
    */
   public final void redispatchEvent (Component target, AWTEvent e)
   {
-    synchronized (e)
-      {
-        e.setSource (target);
-        target.dispatchEvent (e);
-      }
+    e.isFocusManagerEvent = true;
+    target.dispatchEvent (e);
+    e.isFocusManagerEvent = false;
   }
 
   /**
@@ -1364,11 +1317,11 @@ public abstract class KeyboardFocusManager
    *
    * @return a global object set by the current ThreadGroup, or null
    *
-   * @see getFocusOwner
-   * @see getPermanentFocusOwner
-   * @see getFocusedWindow
-   * @see getActiveWindow
-   * @see getCurrentFocusCycleRoot
+   * @see #getFocusOwner()
+   * @see #getPermanentFocusOwner()
+   * @see #getFocusedWindow()
+   * @see #getActiveWindow()
+   * @see #getCurrentFocusCycleRoot()
    */
   private Object getObject (Map globalMap)
   {
@@ -1388,23 +1341,25 @@ public abstract class KeyboardFocusManager
    * @throws SecurityException if this is not the keyboard focus
    * manager associated with the current {@link java.lang.ThreadGroup}
    *
-   * @see getGlobalFocusOwner
-   * @see getGlobalPermanentFocusOwner
-   * @see getGlobalFocusedWindow
-   * @see getGlobalActiveWindow
-   * @see getGlobalCurrentFocusCycleRoot
+   * @see #getGlobalFocusOwner()
+   * @see #getGlobalPermanentFocusOwner()
+   * @see #getGlobalFocusedWindow()
+   * @see #getGlobalActiveWindow()
+   * @see #getGlobalCurrentFocusCycleRoot()
    */
-  private Object getGlobalObject (Map globalMap)
+  private Object getGlobalObject (Map globalMap, boolean checkThread)
   {
-    ThreadGroup currentGroup = Thread.currentThread ().getThreadGroup ();
-    KeyboardFocusManager managerForCallingThread
-      = (KeyboardFocusManager) currentKeyboardFocusManagers.get (currentGroup);
-
-    if (this != managerForCallingThread)
-      throw new SecurityException ("Attempted to retrieve an object from a "
-                                   + "keyboard focus manager that isn't "
-                                   + "associated with the current thread group.");
+    if (checkThread)
+      {
+        ThreadGroup currentGroup = Thread.currentThread ().getThreadGroup ();
+        KeyboardFocusManager managerForCallingThread =
+         (KeyboardFocusManager) currentKeyboardFocusManagers.get(currentGroup);
 
+        if (this != managerForCallingThread)
+          throw new SecurityException ("Attempted to retrieve an object from a "
+                                       + "keyboard focus manager that isn't "
+                                + "associated with the current thread group.");
+      }
     synchronized (globalMap)
       {
         Collection globalObjects = globalMap.values ();
@@ -1432,11 +1387,11 @@ public abstract class KeyboardFocusManager
    * @param newObject the object to set
    * @param property the property that will change
    *
-   * @see setGlobalFocusOwner
-   * @see setGlobalPermanentFocusOwner
-   * @see setGlobalFocusedWindow
-   * @see setGlobalActiveWindow
-   * @see setGlobalCurrentFocusCycleRoot
+   * @see #setGlobalFocusOwner(Component)
+   * @see #setGlobalPermanentFocusOwner(Component)
+   * @see #setGlobalFocusedWindow(Window)
+   * @see #setGlobalActiveWindow(Window)
+   * @see #setGlobalCurrentFocusCycleRoot(Container)
    */
   private void setGlobalObject (Map globalMap,
                                 Object newObject,
@@ -1445,7 +1400,7 @@ public abstract class KeyboardFocusManager
     synchronized (globalMap)
       {
         // Save old object.
-        Object oldObject = getGlobalObject (globalMap);
+        Object oldObject = getGlobalObject(globalMap, false);
 
         // Nullify old object.
         Collection threadGroups = globalMap.keySet ();
@@ -1475,4 +1430,48 @@ public abstract class KeyboardFocusManager
           }
       }
   }
+
+  
+  /**
+   * Maps focus requests from heavyweight to lightweight components.
+   */
+  private static HashMap focusRequests = new HashMap();
+
+  /**
+   * Retargets focus events that come from the peer (which only know about
+   * heavyweight components) to go to the correct lightweight component
+   * if appropriate.
+   *
+   * @param ev the event to check
+   *
+   * @return the retargetted event
+   */
+  static AWTEvent retargetFocusEvent(AWTEvent ev)
+  {
+    if (ev instanceof FocusEvent)
+      {
+        FocusEvent fe = (FocusEvent) ev;
+        Component target = fe.getComponent();
+        if (focusRequests.containsKey(target))
+          {
+            Component lightweight = (Component) focusRequests.get(target);
+            ev = new FocusEvent(lightweight, fe.id, fe.isTemporary());
+            focusRequests.remove(target);
+          }
+      }
+    return ev;
+  }
+
+  /**
+   * Adds a lightweight focus request for a heavyweight component.
+   *
+   * @param heavyweight the heavyweight from which we will receive a focus
+   *        event soon
+   * @param lightweight the lightweight that ultimately receives the request
+   */
+  static void addLightweightFocusRequest(Component heavyweight,
+                                         Component lightweight)
+  {
+    focusRequests.put(heavyweight, lightweight);
+  }
 }
