@@ -1,5 +1,5 @@
 /* Scrollbar.java -- AWT Scrollbar widget
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
@@ -341,17 +341,22 @@ public class Scrollbar extends Component implements Accessible, Adjustable
   public synchronized void setValues(int value, int visibleAmount,
                                      int minimum, int maximum)
   {
-    if (maximum < minimum)
-      maximum = minimum;
+    if (visibleAmount <= 0)
+      visibleAmount = 1;
+    
+    if (maximum <= minimum)
+      maximum = minimum + 1;
 
     if (value < minimum)
       value = minimum;
 
-    if (value > maximum)
-      value = maximum;
-
     if (visibleAmount > maximum - minimum)
       visibleAmount = maximum - minimum;
+    
+    // According to documentation, the actual maximum
+    // value is (maximum - visibleAmount)
+    if (value > maximum - visibleAmount)
+      value = maximum - visibleAmount;
 
     ScrollbarPeer peer = (ScrollbarPeer) getPeer();
     if (peer != null
@@ -362,30 +367,7 @@ public class Scrollbar extends Component implements Accessible, Adjustable
     this.value = value;
     this.visibleAmount = visibleAmount;
     this.minimum = minimum;
-    this.maximum = maximum;
-
-    int range = maximum - minimum;
-    if (lineIncrement > range)
-      {
-	if (range == 0)
-	  lineIncrement = 1;
-	else
-	  lineIncrement = range;
-
-	if (peer != null)
-	  peer.setLineIncrement(lineIncrement);
-      }
-
-    if (pageIncrement > range)
-      {
-	if (range == 0)
-	  pageIncrement = 1;
-	else
-	  pageIncrement = range;
-
-	if (peer != null)
-	  peer.setPageIncrement(pageIncrement);
-      }
+    this.maximum = maximum;      
   }
 
   /**
@@ -437,19 +419,13 @@ public class Scrollbar extends Component implements Accessible, Adjustable
   {
     if (lineIncrement < 0)
       throw new IllegalArgumentException("Unit increment less than zero.");
-
-    int range = maximum - minimum;
-    if (lineIncrement > range)
-      {
-	if (range == 0)
-	  lineIncrement = 1;
-	else
-	  lineIncrement = range;
-      }
-
-    if (lineIncrement == this.lineIncrement)
+    
+    if (lineIncrement == 0)
+      lineIncrement = 1;
+         
+   if (lineIncrement == this.lineIncrement)
       return;
-
+    
     this.lineIncrement = lineIncrement;
 
     ScrollbarPeer peer = (ScrollbarPeer) getPeer();
@@ -507,15 +483,9 @@ public class Scrollbar extends Component implements Accessible, Adjustable
     if (pageIncrement < 0)
       throw new IllegalArgumentException("Block increment less than zero.");
 
-    int range = maximum - minimum;
-    if (pageIncrement > range)
-      {
-	if (range == 0)
-	  pageIncrement = 1;
-	else
-	  pageIncrement = range;
-      }
-
+    if (pageIncrement == 0)
+      pageIncrement = 1;
+    
     if (pageIncrement == this.pageIncrement)
       return;
 
@@ -593,13 +563,33 @@ public class Scrollbar extends Component implements Accessible, Adjustable
       adjustment_listeners.adjustmentValueChanged(event);
   }
 
+  /**
+   * Package private method to determine whether to call
+   * processEvent() or not.  Will handle events from peer and update
+   * the current value.
+   */
   void dispatchEventImpl(AWTEvent e)
   {
     if (e.id <= AdjustmentEvent.ADJUSTMENT_LAST 
-      && e.id >= AdjustmentEvent.ADJUSTMENT_FIRST
-      && (adjustment_listeners != null 
-	  || (eventMask & AWTEvent.ADJUSTMENT_EVENT_MASK) != 0))
-      processEvent(e);
+	&& e.id >= AdjustmentEvent.ADJUSTMENT_FIRST)
+      {
+	AdjustmentEvent ae = (AdjustmentEvent) e;
+	boolean adjusting = ae.getValueIsAdjusting();
+	if (adjusting)
+	  setValueIsAdjusting(true);
+	try
+	  {
+	    setValue(((AdjustmentEvent) e).getValue());
+	    if (adjustment_listeners != null 
+		|| (eventMask & AWTEvent.ADJUSTMENT_EVENT_MASK) != 0)
+	      processEvent(e);
+	  }
+	finally
+	  {
+	    if (adjusting)
+	      setValueIsAdjusting(false);
+	  }
+      }
     else
       super.dispatchEventImpl(e);
   }
