@@ -38,11 +38,7 @@ exception statement from your version. */
 
 package java.awt;
 
-import gnu.classpath.NotImplementedException;
-
 import java.awt.event.ComponentEvent;
-import java.awt.event.FocusEvent;
-import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.awt.event.WindowFocusListener;
 import java.awt.event.WindowListener;
@@ -82,6 +78,8 @@ public class Window extends Container implements Accessible
   private int state = 0;
   /** @since 1.4 */
   private boolean focusableWindowState = true;
+  /** @since 1.5 */
+  private boolean alwaysOnTop = false;
 
   // A list of other top-level windows owned by this window.
   private transient Vector ownedWindows = new Vector();
@@ -132,33 +130,6 @@ public class Window extends Container implements Accessible
     // cycle roots.
     focusCycleRoot = true;
     setLayout(new BorderLayout());
-
-    addWindowFocusListener (new WindowAdapter ()
-      {
-        public void windowGainedFocus (WindowEvent event)
-        {
-          if (windowFocusOwner != null)
-            {
-              // FIXME: move this section and the other similar
-              // sections in Component into a separate method.
-              EventQueue eq = Toolkit.getDefaultToolkit ().getSystemEventQueue ();
-              synchronized (eq)
-                {
-                  KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager ();
-                  Component currentFocusOwner = manager.getGlobalPermanentFocusOwner ();
-                  if (currentFocusOwner != null)
-                    {
-                      eq.postEvent (new FocusEvent (currentFocusOwner, FocusEvent.FOCUS_LOST,
-                                                    false, windowFocusOwner));
-                      eq.postEvent (new FocusEvent (windowFocusOwner, FocusEvent.FOCUS_GAINED,
-                                                    false, currentFocusOwner));
-                    }
-                  else
-                    eq.postEvent (new FocusEvent (windowFocusOwner, FocusEvent.FOCUS_GAINED, false));
-                }
-            }
-        }
-      });
     
     GraphicsEnvironment g = GraphicsEnvironment.getLocalGraphicsEnvironment();
     graphicsConfiguration = g.getDefaultScreenDevice().getDefaultConfiguration();
@@ -284,8 +255,6 @@ public class Window extends Container implements Accessible
   {
     synchronized (getTreeLock())
       {
-        if (parent != null && ! parent.isDisplayable())
-          parent.addNotify();
         if (peer == null)
           addNotify();
 
@@ -327,6 +296,15 @@ public class Window extends Container implements Accessible
             if (initialFocusOwner != null)
               initialFocusOwner.requestFocusInWindow();
 
+            // Post WINDOW_OPENED from here.
+            if (windowListener != null
+                || (eventMask & AWTEvent.WINDOW_EVENT_MASK) != 0)
+              {
+                WindowEvent ev = new WindowEvent(this,
+                                                 WindowEvent.WINDOW_OPENED);
+                Toolkit tk = Toolkit.getDefaultToolkit();
+                tk.getSystemEventQueue().postEvent(ev);
+              }
             shown = true;
           }
       }
@@ -378,22 +356,32 @@ public class Window extends Container implements Accessible
 	  component[i].removeNotify();
 	this.removeNotify();
 
-        // Post a WINDOW_CLOSED event.
-        WindowEvent we = new WindowEvent(this, WindowEvent.WINDOW_CLOSED);
-        getToolkit().getSystemEventQueue().postEvent(we);
+	// Post WINDOW_CLOSED from here.
+	if (windowListener != null
+	    || (eventMask & AWTEvent.WINDOW_EVENT_MASK) != 0)
+	  {
+	    WindowEvent ev = new WindowEvent(this,
+	                                     WindowEvent.WINDOW_CLOSED);
+	    Toolkit tk = Toolkit.getDefaultToolkit();
+	    tk.getSystemEventQueue().postEvent(ev);
+	  }
       }
   }
 
   /**
    * Sends this window to the back so that all other windows display in
    * front of it.
+   *
+   * If the window is set to be always-on-top, this will remove its
+   * always-on-top status.
    */
   public void toBack()
   {
     if (peer != null)
       {
-	WindowPeer wp = (WindowPeer) peer;
-	wp.toBack();
+	if( alwaysOnTop )
+	  setAlwaysOnTop( false );
+	( (WindowPeer) peer ).toBack();
       }
   }
 
@@ -404,10 +392,7 @@ public class Window extends Container implements Accessible
   public void toFront()
   {
     if (peer != null)
-      {
-        WindowPeer wp = (WindowPeer) peer;
-        wp.toFront();
-      }
+      ( (WindowPeer) peer ).toFront();
   }
 
   /**
@@ -507,7 +492,11 @@ public class Window extends Container implements Accessible
    */
   public synchronized void addWindowListener(WindowListener listener)
   {
-    windowListener = AWTEventMulticaster.add(windowListener, listener);
+    if (listener != null)
+      {
+        newEventsOnly = true;
+        windowListener = AWTEventMulticaster.add(windowListener, listener);
+      }
   }
 
   /**
@@ -564,7 +553,12 @@ public class Window extends Container implements Accessible
    */
   public void addWindowFocusListener (WindowFocusListener wfl)
   {
-    windowFocusListener = AWTEventMulticaster.add (windowFocusListener, wfl);
+    if (wfl != null)
+      {
+        newEventsOnly = true;
+        windowFocusListener = AWTEventMulticaster.add (windowFocusListener,
+                                                       wfl);
+      }
   }
   
   /**
@@ -574,7 +568,12 @@ public class Window extends Container implements Accessible
    */
   public void addWindowStateListener (WindowStateListener wsl)
   {
-    windowStateListener = AWTEventMulticaster.add (windowStateListener, wsl);  
+    if (wsl != null)
+      {
+        newEventsOnly = true;
+        windowStateListener = AWTEventMulticaster.add (windowStateListener,
+                                                       wsl);  
+      }
   }
   
   /**
@@ -614,33 +613,12 @@ public class Window extends Container implements Accessible
 
   void dispatchEventImpl(AWTEvent e)
   {
-    // Make use of event id's in order to avoid multiple instanceof tests.
-    if (e.id <= WindowEvent.WINDOW_LAST 
-        && e.id >= WindowEvent.WINDOW_FIRST
-        && (windowListener != null
-	    || windowFocusListener != null
-	    || windowStateListener != null
-	    || (eventMask & AWTEvent.WINDOW_EVENT_MASK) != 0))
-      processEvent(e);
-    else 
+    if (e.getID() == ComponentEvent.COMPONENT_RESIZED)
       {
-	if (peer != null && (e.id == ComponentEvent.COMPONENT_RESIZED
-	    || e.id == ComponentEvent.COMPONENT_MOVED))
-            {
-	    Rectangle bounds = peer.getBounds();
-	    x = bounds.x;
-	    y = bounds.y;
-	    height = bounds.height;
-	    width = bounds.width;
-	    
-	    if (e.id == ComponentEvent.COMPONENT_RESIZED)
-	      {
-		invalidate();
-		validate();
-	      }
-	  }
-	super.dispatchEventImpl(e);
+        invalidate();
+        validate();
       }
+    super.dispatchEventImpl(e);
   }
 
   /**
@@ -654,7 +632,28 @@ public class Window extends Container implements Accessible
   protected void processEvent(AWTEvent evt)
   {
     if (evt instanceof WindowEvent)
-      processWindowEvent((WindowEvent) evt);
+      {
+        WindowEvent we = (WindowEvent) evt;
+        switch (evt.getID())
+          {
+          case WindowEvent.WINDOW_OPENED:
+          case WindowEvent.WINDOW_CLOSED:
+          case WindowEvent.WINDOW_CLOSING:
+          case WindowEvent.WINDOW_ICONIFIED:
+          case WindowEvent.WINDOW_DEICONIFIED:
+          case WindowEvent.WINDOW_ACTIVATED:
+          case WindowEvent.WINDOW_DEACTIVATED:
+            processWindowEvent(we);
+            break;
+          case WindowEvent.WINDOW_GAINED_FOCUS:
+          case WindowEvent.WINDOW_LOST_FOCUS:
+            processWindowFocusEvent(we);
+            break;
+          case WindowEvent.WINDOW_STATE_CHANGED:
+            processWindowStateEvent(we);
+            break;
+          }
+      }
     else
       super.processEvent(evt);
   }
@@ -669,54 +668,35 @@ public class Window extends Container implements Accessible
    */
   protected void processWindowEvent(WindowEvent evt)
   {
-    int id = evt.getID();
-
-    if (id == WindowEvent.WINDOW_GAINED_FOCUS
-	|| id == WindowEvent.WINDOW_LOST_FOCUS)
-      processWindowFocusEvent (evt);
-    else if (id == WindowEvent.WINDOW_STATE_CHANGED)
-      processWindowStateEvent (evt);
-    else
+    if (windowListener != null)
       {
-	if (windowListener != null)
-	  {
-	    switch (evt.getID())
-	      {
-	      case WindowEvent.WINDOW_ACTIVATED:
-		windowListener.windowActivated(evt);
-		break;
-
-	      case WindowEvent.WINDOW_CLOSED:
-		windowListener.windowClosed(evt);
-		break;
-
-	      case WindowEvent.WINDOW_CLOSING:
-		windowListener.windowClosing(evt);
-		break;
-
-	      case WindowEvent.WINDOW_DEACTIVATED:
-		windowListener.windowDeactivated(evt);
-		break;
-
-	      case WindowEvent.WINDOW_DEICONIFIED:
-		windowListener.windowDeiconified(evt);
-		break;
-
-	      case WindowEvent.WINDOW_ICONIFIED:
-		windowListener.windowIconified(evt);
-		break;
-
-	      case WindowEvent.WINDOW_OPENED:
-		windowListener.windowOpened(evt);
-		break;
-
-	      default:
-		break;
-	      }
-	  }
+        switch (evt.getID())
+          {
+          case WindowEvent.WINDOW_ACTIVATED:
+            windowListener.windowActivated(evt);
+            break;
+          case WindowEvent.WINDOW_CLOSED:
+            windowListener.windowClosed(evt);
+            break;
+          case WindowEvent.WINDOW_CLOSING:
+            windowListener.windowClosing(evt);
+            break;
+          case WindowEvent.WINDOW_DEACTIVATED:
+            windowListener.windowDeactivated(evt);
+            break;
+          case WindowEvent.WINDOW_DEICONIFIED:
+            windowListener.windowDeiconified(evt);
+            break;
+          case WindowEvent.WINDOW_ICONIFIED:
+            windowListener.windowIconified(evt);
+            break;
+          case WindowEvent.WINDOW_OPENED:
+            windowListener.windowOpened(evt);
+            break;
+          }
       }
   }
-  
+
   /**
    * Identifies if this window is active.  The active window is a Frame or
    * Dialog that has focus or owns the active window.
@@ -1046,12 +1026,11 @@ public class Window extends Container implements Accessible
   /**
    * @since 1.2
    *
-   * @deprecated
+   * @deprecated replaced by Component.applyComponentOrientation.
    */
   public void applyResourceBundle(ResourceBundle rb)
-    throws NotImplementedException
   {
-    throw new Error ("Not implemented");
+    applyComponentOrientation(ComponentOrientation.getOrientation(rb));
   }
 
   /**
@@ -1203,6 +1182,55 @@ public class Window extends Container implements Accessible
     return null;
   }
 
+  /**
+   * Returns whether the Windows is an always-on-top window,
+   * meaning whether the window can be obscured by other windows or not.
+   *
+   * @return <code>true</code> if the windows is always-on-top,
+   * <code>false</code> otherwise.
+   * @since 1.5
+   */
+  public final boolean isAlwaysOnTop()
+  {
+    return alwaysOnTop;
+  }
+
+  /**
+   * Sets the always-on-top state of this window (if supported).
+   *
+   * Setting a window to always-on-top means it will not be obscured
+   * by any other windows (with the exception of other always-on-top 
+   * windows). Not all platforms may support this.
+   *
+   * If an window's always-on-top status is changed to false, the window
+   * will remain at the front but not be anchored there.
+   *
+   * Calling toBack() on an always-on-top window will change its
+   * always-on-top status to false.
+   *
+   * @since 1.5
+   */
+  public final void setAlwaysOnTop(boolean alwaysOnTop)
+  {
+    SecurityManager sm = System.getSecurityManager();
+    if (sm != null)
+      sm.checkPermission( new AWTPermission("setWindowAlwaysOnTop") );
+
+    if( this.alwaysOnTop == alwaysOnTop )
+      return;
+    
+    if( alwaysOnTop )
+      toFront();
+
+    firePropertyChange("alwaysOnTop", this.alwaysOnTop, alwaysOnTop );
+    this.alwaysOnTop = alwaysOnTop;
+
+    if (peer != null) 
+      ( (WindowPeer) peer).updateAlwaysOnTop();
+    else
+      System.out.println("Null peer?!");
+  }
+
   /**
    * Generate a unique name for this window.
    *
@@ -1213,6 +1241,42 @@ public class Window extends Container implements Accessible
     return "win" + getUniqueLong();
   }
 
+  /**
+   * Overridden to handle WindowEvents.
+   *
+   * @return <code>true</code> when the specified event type is enabled,
+   *         <code>false</code> otherwise
+   */
+  boolean eventTypeEnabled(int type)
+  {
+    boolean enabled = false;
+    switch (type)
+      {
+        case WindowEvent.WINDOW_OPENED:
+        case WindowEvent.WINDOW_CLOSED:
+        case WindowEvent.WINDOW_CLOSING:
+        case WindowEvent.WINDOW_ICONIFIED:
+        case WindowEvent.WINDOW_DEICONIFIED:
+        case WindowEvent.WINDOW_ACTIVATED:
+        case WindowEvent.WINDOW_DEACTIVATED:
+          enabled = ((eventMask & AWTEvent.WINDOW_EVENT_MASK) != 0)
+                    || windowListener != null;
+          break;
+        case WindowEvent.WINDOW_GAINED_FOCUS:
+        case WindowEvent.WINDOW_LOST_FOCUS:
+          enabled = ((eventMask & AWTEvent.WINDOW_FOCUS_EVENT_MASK) != 0)
+                    || windowFocusListener != null;
+          break;
+        case WindowEvent.WINDOW_STATE_CHANGED:
+          enabled = ((eventMask & AWTEvent.WINDOW_STATE_EVENT_MASK) != 0)
+                    || windowStateListener != null;
+          break;
+        default:
+          enabled = super.eventTypeEnabled(type);
+      }
+    return enabled;
+  }
+
   private static synchronized long getUniqueLong()
   {
     return next_window_number++;
