@@ -16,8 +16,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -202,13 +202,41 @@ public class MulticastSocket extends DatagramSocket
   {
     if (isClosed())
       throw new SocketException("socket is closed");
-
-    Enumeration e = netIf.getInetAddresses();
-
-    if (! e.hasMoreElements())
-      throw new SocketException("no network devices found");
-
-    InetAddress address = (InetAddress) e.nextElement();
+    
+    InetAddress address;
+    if (netIf != null)
+      out:
+      {
+        Enumeration e = netIf.getInetAddresses();
+        if (getLocalAddress() instanceof Inet4Address)
+          {
+            // Search for a IPv4 address.
+            while (e.hasMoreElements())
+              {
+                address = (InetAddress) e.nextElement();
+                if (address instanceof Inet4Address)
+                  break out;
+              }
+            throw new SocketException("interface " + netIf.getName() + " has no IPv6 address");
+          }
+        else if (getLocalAddress() instanceof Inet6Address)
+          {
+            // Search for a IPv6 address.
+            while (e.hasMoreElements())
+              {
+                address = (InetAddress) e.nextElement();
+                if (address instanceof Inet6Address)
+                  break out;
+              }
+            throw new SocketException("interface " + netIf.getName() + " has no IPv6 address");
+          }
+        else
+          throw new SocketException("interface " + netIf.getName() + " has no suitable IP address");
+      }
+    else
+      address = InetAddress.ANY_IF;
+    
+    
     getImpl().setOption(SocketOptions.IP_MULTICAST_IF, address);
   }
 
@@ -230,6 +258,10 @@ public class MulticastSocket extends DatagramSocket
 
     InetAddress address =
       (InetAddress) getImpl().getOption(SocketOptions.IP_MULTICAST_IF);
+    
+    if (address.isAnyLocalAddress())
+      return NetworkInterface.createAnyInterface();
+    
     NetworkInterface netIf = NetworkInterface.getByInetAddress(address);
 
     return netIf;
