@@ -1,5 +1,5 @@
 /* NetworkInterface.java --
-   Copyright (C) 2002, 2003, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -38,7 +38,14 @@ exception statement from your version. */
 
 package java.net;
 
+import gnu.classpath.SystemProperties;
+
+import java.util.Collection;
+import java.util.Collections;
 import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.Map;
 import java.util.Vector;
 
 /**
@@ -53,25 +60,23 @@ import java.util.Vector;
  */
 public final class NetworkInterface
 {
-  private String name;
-  private Vector inetAddresses;
+  private final VMNetworkInterface netif;
 
-  NetworkInterface(String name, InetAddress address)
+  private NetworkInterface(VMNetworkInterface netif)
   {
-    this.name = name;
-    this.inetAddresses = new Vector(1, 1);
-    this.inetAddresses.add(address);
+    this.netif = netif;
   }
-
-  NetworkInterface(String name, InetAddress[] addresses)
+  
+  /** Creates an NetworkInterface instance which
+   * represents any interface in the system. Its only
+   * address is <code>0.0.0.0/0.0.0.0</code>. This
+   * method is needed by {@link MulticastSocket#getNetworkInterface}
+   */
+  static NetworkInterface createAnyInterface()
   {
-    this.name = name;
-    this.inetAddresses = new Vector(addresses.length, 1);
-
-    for (int i = 0; i < addresses.length; i++)
-      this.inetAddresses.add(addresses[i]);
+    return new NetworkInterface(new VMNetworkInterface());
   }
-
+  
   /**
    * Returns the name of the network interface
    *
@@ -79,7 +84,7 @@ public final class NetworkInterface
    */
   public String getName()
   {
-    return name;
+    return netif.name;
   }
 
   /**
@@ -95,6 +100,7 @@ public final class NetworkInterface
   public Enumeration getInetAddresses()
   {
     SecurityManager s = System.getSecurityManager();
+    Vector inetAddresses = new Vector(netif.addresses);
 
     if (s == null)
       return inetAddresses.elements();
@@ -107,7 +113,7 @@ public final class NetworkInterface
 	InetAddress addr = (InetAddress) addresses.nextElement();
 	try
 	  {
-	    s.checkConnect(addr.getHostAddress(), 58000);
+	    s.checkConnect(addr.getHostAddress(), -1);
 	    tmpInetAddresses.add(addr);
 	  }
 	catch (SecurityException e)
@@ -126,7 +132,7 @@ public final class NetworkInterface
    */
   public String getDisplayName()
   {
-    return name;
+    return netif.name;
   }
 
   /**
@@ -143,17 +149,14 @@ public final class NetworkInterface
   public static NetworkInterface getByName(String name)
     throws SocketException
   {
-    Vector networkInterfaces = VMNetworkInterface.getInterfaces();
-
-    for (Enumeration e = networkInterfaces.elements(); e.hasMoreElements();)
+    if (name == null)
+      throw new NullPointerException();
+    VMNetworkInterface[] netifs = VMNetworkInterface.getVMInterfaces();
+    for (int i = 0; i < netifs.length; i++)
       {
-	NetworkInterface tmp = (NetworkInterface) e.nextElement();
-
-	if (name.equals(tmp.getName()))
-	  return tmp;
+        if (netifs[i].name.equals(name))
+          return new NetworkInterface(netifs[i]);
       }
-
-    // No interface with the given name found.
     return null;
   }
 
@@ -170,22 +173,15 @@ public final class NetworkInterface
   public static NetworkInterface getByInetAddress(InetAddress addr)
     throws SocketException
   {
-    Vector networkInterfaces = VMNetworkInterface.getInterfaces();
-
-    for (Enumeration interfaces = networkInterfaces.elements();
-         interfaces.hasMoreElements();)
+    if (addr == null)
+      throw new NullPointerException();
+    VMNetworkInterface[] netifs = VMNetworkInterface.getVMInterfaces();
+    for (int i = 0; i < netifs.length; i++)
       {
-	NetworkInterface tmp = (NetworkInterface) interfaces.nextElement();
-
-	for (Enumeration addresses = tmp.inetAddresses.elements();
-	     addresses.hasMoreElements();)
-	  {
-	    if (addr.equals((InetAddress) addresses.nextElement()))
-	      return tmp;
-	  }
+        if (netifs[i].addresses.contains(addr))
+          return new NetworkInterface(netifs[i]);
       }
-
-    throw new SocketException("no network interface is bound to such an IP address");
+    return null;
   }
 
   /**
@@ -197,11 +193,13 @@ public final class NetworkInterface
    */
   public static Enumeration getNetworkInterfaces() throws SocketException
   {
-    Vector networkInterfaces = VMNetworkInterface.getInterfaces();
-
-    if (networkInterfaces.isEmpty())
-      return null;
-
+    VMNetworkInterface[] netifs = VMNetworkInterface.getVMInterfaces();
+    Vector networkInterfaces = new Vector(netifs.length);
+    for (int i = 0; i < netifs.length; i++)
+      {
+        if (!netifs[i].addresses.isEmpty())
+          networkInterfaces.add(new NetworkInterface(netifs[i]));
+      }
     return networkInterfaces.elements();
   }
 
@@ -218,8 +216,12 @@ public final class NetworkInterface
       return false;
 
     NetworkInterface tmp = (NetworkInterface) obj;
+    
+    if (netif.name == null)
+      return tmp.netif.name == null;
 
-    return (name.equals(tmp.name) && inetAddresses.equals(tmp.inetAddresses));
+    return (netif.name.equals(tmp.netif.name)
+            && (netif.addresses.equals(tmp.netif.addresses)));
   }
 
   /**
@@ -230,7 +232,12 @@ public final class NetworkInterface
   public int hashCode()
   {
     // FIXME: hash correctly
-    return name.hashCode() + inetAddresses.hashCode();
+    int hc = netif.addresses.hashCode();
+    
+    if (netif.name != null)
+      hc += netif.name.hashCode();
+    
+    return hc;
   }
 
   /**
@@ -241,19 +248,22 @@ public final class NetworkInterface
   public String toString()
   {
     // FIXME: check if this is correct
-    String result;
-    String separator = System.getProperty("line.separator");
+    StringBuffer result;
+    String separator = SystemProperties.getProperty("line.separator");
 
-    result =
-      "name: " + getDisplayName() + " (" + getName() + ") addresses:"
-      + separator;
+    result = new StringBuffer();
+    
+    result.append("name: ");
+    result.append(getDisplayName());
+    result.append(" (").append(getName()).append(") addresses:");
+    result.append(separator);
 
-    for (Enumeration e = inetAddresses.elements(); e.hasMoreElements();)
+    for (Iterator it = netif.addresses.iterator(); it.hasNext(); )
       {
-	InetAddress address = (InetAddress) e.nextElement();
-	result += address.toString() + ";" + separator;
+	InetAddress address = (InetAddress) it.next();
+	result.append(address.toString()).append(";").append(separator);
       }
 
-    return result;
+    return result.toString();
   }
 }
