@@ -218,18 +218,18 @@ public abstract class NumberFormat extends Format implements Cloneable
   public final String format (long number)
   {
     StringBuffer sbuf = new StringBuffer(50);
-    format (number, sbuf, null);
+    format (number, sbuf, new FieldPosition(0));
     return sbuf.toString();
   }
 
-  public final StringBuffer format (Object obj, StringBuffer sbuf,
+  public StringBuffer format (Object obj, StringBuffer sbuf,
 				    FieldPosition pos)
   {
     if (obj instanceof Number)
       return format(((Number) obj).doubleValue(), sbuf, pos);
-    else
-      throw new IllegalArgumentException 
-	("Cannot format given Object as a Number");
+  
+    throw new
+      IllegalArgumentException("Cannot format given Object as a Number");
   }
 
   /**
@@ -351,7 +351,7 @@ public abstract class NumberFormat extends Format implements Cloneable
   {
     NumberFormat format;
     
-    format = computeInstance (loc, "currencyFormat", "$#,##0.00;($#,##0.00)");
+    format = computeInstance (loc, "currencyFormat", "\u00A4#,##0.00;(\u00A4#,##0.00)");
     format.setMaximumFractionDigits(format.getCurrency().getDefaultFractionDigits());  
     return format;
   }
@@ -481,7 +481,7 @@ public abstract class NumberFormat extends Format implements Cloneable
    */
   public static NumberFormat getIntegerInstance(Locale locale)
   {
-    NumberFormat format = computeInstance (locale, "numberFormat", "#,##0");
+    NumberFormat format = computeInstance (locale, "integerFormat", "#,##0");
     format.setMaximumFractionDigits(0);
     format.setParseIntegerOnly (true);
     return format;
@@ -720,7 +720,9 @@ public abstract class NumberFormat extends Format implements Cloneable
   public final String format (double number)
   {
     StringBuffer sbuf = new StringBuffer(50);
-    format (number, sbuf, null);
+    FieldPosition position = new FieldPosition(0);
+    
+    format (number, sbuf, position);
     return sbuf.toString();
   }
 
