@@ -1,5 +1,5 @@
 /* EventListenerList.java --
-   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005, 2006,  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -37,6 +37,9 @@ exception statement from your version. */
 
 package javax.swing.event;
 
+import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.io.ObjectOutputStream;
 import java.io.Serializable;
 import java.lang.reflect.Array;
 import java.util.EventListener;
@@ -108,7 +111,7 @@ public class EventListenerList
    * An array with all currently registered listeners.  The array has
    * twice as many elements as there are listeners.  For an even
    * integer <code>i</code>, <code>listenerList[i]</code> indicates
-   * the registered class, and <code>listenerList[i+1]</code> is the
+   * the registered class, and <code>listenerList[i + 1]</code> is the
    * listener.
    */
   protected transient Object[] listenerList = NO_LISTENERS;
@@ -187,11 +190,15 @@ public class EventListenerList
 
 
   /**
-   * Get a list of listenerType/listener pairs
-   * @return Listener list
+   * Returns an array containing a sequence of listenerType/listener pairs, one
+   * for each listener.
+   * 
+   * @return An array containing the listener types and references.
    */
   public Object[] getListenerList()
   {
+    // returning the internal storage is a bad idea, but tests show that the
+    // reference implementation does this...
     return listenerList;
   }
 
@@ -300,4 +307,51 @@ public class EventListenerList
       }
     return buf.toString();
   }
+
+  /**
+   * Serializes an instance to an ObjectOutputStream.
+   *
+   * @param out the stream to serialize to
+   *
+   * @throws IOException if something goes wrong
+   */
+  private void writeObject(ObjectOutputStream out)
+    throws IOException
+  {
+    out.defaultWriteObject();
+    for (int i = 0; i < listenerList.length; i += 2)
+      {
+        Class cl = (Class) listenerList[i];
+        EventListener l = (EventListener) listenerList[i + 1];
+        if (l != null && l instanceof Serializable)
+          {
+            out.writeObject(cl.getName());
+            out.writeObject(l);
+          }
+      }
+    // Write end marker.
+    out.writeObject(null);
+  }
+
+  /**
+   * Deserializes an instance from an ObjectInputStream.
+   *
+   * @param in the input stream
+   *
+   * @throws ClassNotFoundException if a serialized class can't be found
+   * @throws IOException if something goes wrong
+   */
+  private void readObject(ObjectInputStream in)
+    throws ClassNotFoundException, IOException
+  {
+    listenerList = NO_LISTENERS;
+    in.defaultReadObject();
+    Object type;
+    ClassLoader cl = Thread.currentThread().getContextClassLoader();
+    while ((type = in.readObject()) != null)
+      {
+        EventListener l = (EventListener) in.readObject();
+        add(Class.forName((String) type, true, cl), l);
+      }
+  }
 }
