@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package javax.swing.plaf.metal;
 
+import gnu.classpath.SystemProperties;
+
 import java.awt.Color;
 import java.awt.Font;
 
@@ -67,7 +69,7 @@ import javax.swing.plaf.basic.BasicLookAndFeel;
  * }</pre>
  */
 public class MetalLookAndFeel extends BasicLookAndFeel
-{	   
+{          
   private static final long serialVersionUID = 6680646159193457980L;
   
   /** The current theme. */
@@ -81,7 +83,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
    */
   public MetalLookAndFeel()
   {
-    createDefaultTheme();
+    // Nothing to do here.
   }
 
   /**
@@ -89,8 +91,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
    */
   protected void createDefaultTheme()
   {
-    if (theme == null)
-      setCurrentTheme(new OceanTheme());
+    getCurrentTheme();
   }
 
   /**
@@ -149,6 +150,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
 
   public UIDefaults getDefaults()
   {
+    createDefaultTheme();
     if (LAF_defaults == null)
       {
         LAF_defaults = super.getDefaults();
@@ -709,6 +711,8 @@ public class MetalLookAndFeel extends BasicLookAndFeel
    * @param theme  the theme (<code>null</code> not permitted).
    * 
    * @throws NullPointerException if <code>theme</code> is <code>null</code>.
+   * 
+   * @see #getCurrentTheme()
    */
   public static void setCurrentTheme(MetalTheme theme)
   {
@@ -885,14 +889,12 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "CheckBox.border", MetalBorders.getButtonBorder(),
       "CheckBox.disabledText", getInactiveControlTextColor(),
       "CheckBox.focus", getFocusColor(),
-      "CheckBox.font", new FontUIResource("Dialog", Font.BOLD, 12),
+      "CheckBox.font", getControlTextFont(),
       "CheckBox.foreground", getControlTextColor(),
       "CheckBox.icon",
-      new UIDefaults.ProxyLazyValue
-          ("javax.swing.plaf.metal.MetalCheckBoxIcon"),
+      new UIDefaults.ProxyLazyValue("javax.swing.plaf.metal.MetalCheckBoxIcon"),
       "CheckBox.checkIcon",
-      new UIDefaults.ProxyLazyValue
-      ("javax.swing.plaf.metal.MetalCheckBoxIcon"),
+      new UIDefaults.ProxyLazyValue("javax.swing.plaf.metal.MetalCheckBoxIcon"),
       "Checkbox.select", getControlShadow(),
 
       "CheckBoxMenuItem.acceleratorFont", new FontUIResource("Dialog", Font.PLAIN, 10),
@@ -903,7 +905,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "CheckBoxMenuItem.commandSound", "sounds/MenuItemCommand.wav",
       "CheckBoxMenuItem.checkIcon", MetalIconFactory.getCheckBoxMenuItemIcon(),
       "CheckBoxMenuItem.disabledForeground", getMenuDisabledForeground(),
-      "CheckBoxMenuItem.font", new FontUIResource("Dialog", Font.BOLD, 12),
+      "CheckBoxMenuItem.font", getMenuTextFont(),
       "CheckBoxMenuItem.foreground", getMenuForeground(),
       "CheckBoxMenuItem.selectionBackground", getMenuSelectedBackground(),
       "CheckBoxMenuItem.selectionForeground", getMenuSelectedForeground(),
@@ -922,7 +924,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "ComboBox.buttonShadow", getControlShadow(),
       "ComboBox.disabledBackground", getControl(),
       "ComboBox.disabledForeground", getInactiveSystemTextColor(),
-      "ComboBox.font", new FontUIResource("Dialog", Font.BOLD, 12),
+      "ComboBox.font", getControlTextFont(),
       "ComboBox.foreground", getControlTextColor(),
       "ComboBox.selectionBackground", getPrimaryControlShadow(),
       "ComboBox.selectionForeground", getControlTextColor(),
@@ -933,10 +935,11 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "DesktopIcon.foreground", getControlTextColor(),
       "DesktopIcon.width", new Integer(160),
       "DesktopIcon.border", MetalBorders.getDesktopIconBorder(),
+      "DesktopIcon.font", getControlTextFont(),
 
       "EditorPane.background", getWindowBackground(),
       "EditorPane.caretForeground", getUserTextColor(),
-      "EditorPane.font", new FontUIResource("Dialog", Font.BOLD, 12),
+      "EditorPane.font", getControlTextFont(),
       "EditorPane.foreground",  getUserTextColor(),
       "EditorPane.inactiveForeground",  getInactiveSystemTextColor(),
       "EditorPane.selectionBackground", getTextHighlightColor(),
@@ -964,7 +967,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "FileChooser.detailsViewIcon", 
           MetalIconFactory.getFileChooserDetailViewIcon(),
       "FileChooser.fileNameLabelMnemonic", new Integer(78),
-      "FileChooser.filesOfTypeLabelMnemonic",new Integer(84),
+      "FileChooser.filesOfTypeLabelMnemonic", new Integer(84),
       "FileChooser.lookInLabelMnemonic", new Integer(73),
       "FileView.computerIcon", MetalIconFactory.getTreeComputerIcon(),
       "FileView.directoryIcon", MetalIconFactory.getTreeFolderIcon(),
@@ -1021,7 +1024,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "Menu.borderPainted", Boolean.TRUE,
       "MenuItem.commandSound", "sounds/MenuItemCommand.wav",
       "Menu.disabledForeground", getMenuDisabledForeground(),
-      "Menu.font", getControlTextFont(),
+      "Menu.font", getMenuTextFont(),
       "Menu.foreground", getMenuForeground(),
       "Menu.selectionBackground", getMenuSelectedBackground(),
       "Menu.selectionForeground", getMenuSelectedForeground(),
@@ -1030,7 +1033,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
 
       "MenuBar.background", getMenuBackground(),
       "MenuBar.border", new MetalBorders.MenuBarBorder(),
-      "MenuBar.font", getControlTextFont(),
+      "MenuBar.font", getMenuTextFont(),
       "MenuBar.foreground", getMenuForeground(),
       "MenuBar.highlight", getControlHighlight(),
       "MenuBar.shadow", getControlShadow(),
@@ -1044,7 +1047,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "MenuItem.border", new MetalBorders.MenuItemBorder(),
       "MenuItem.borderPainted", Boolean.TRUE,
       "MenuItem.disabledForeground", getMenuDisabledForeground(),
-      "MenuItem.font", getControlTextFont(),
+      "MenuItem.font", getMenuTextFont(),
       "MenuItem.foreground", getMenuForeground(),
       "MenuItem.selectionBackground", getMenuSelectedBackground(),
       "MenuItem.selectionForeground", getMenuSelectedForeground(),
@@ -1085,13 +1088,13 @@ public class MetalLookAndFeel extends BasicLookAndFeel
 
       "PopupMenu.background", getMenuBackground(),
       "PopupMenu.border", new MetalBorders.PopupMenuBorder(),
-      "PopupMenu.font", new FontUIResource("Dialog", Font.BOLD, 12),
+      "PopupMenu.font", getMenuTextFont(),
       "PopupMenu.foreground", getMenuForeground(),
       "PopupMenu.popupSound", "sounds/PopupMenuPopup.wav",
 
       "ProgressBar.background", getControl(),
       "ProgressBar.border", new BorderUIResource.LineBorderUIResource(getControlDarkShadow(), 1),
-      "ProgressBar.font", new FontUIResource("Dialog", Font.BOLD, 12),
+      "ProgressBar.font", getControlTextFont(),
       "ProgressBar.foreground", getPrimaryControlShadow(),
       "ProgressBar.selectionBackground", getPrimaryControlDarkShadow(),
       "ProgressBar.selectionForeground", getControl(),
@@ -1125,7 +1128,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
         MetalIconFactory.getRadioButtonMenuItemIcon(),
       "RadioButtonMenuItem.commandSound", "sounds/MenuItemCommand.wav",
       "RadioButtonMenuItem.disabledForeground", getMenuDisabledForeground(),
-      "RadioButtonMenuItem.font", MetalLookAndFeel.getControlTextFont(),
+      "RadioButtonMenuItem.font", getMenuTextFont(),
       "RadioButtonMenuItem.foreground", getMenuForeground(),
       "RadioButtonMenuItem.margin", new InsetsUIResource(2, 2, 2, 2),
       "RadioButtonMenuItem.selectionBackground", 
@@ -1172,7 +1175,7 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "Spinner.arrowButtonInsets", new InsetsUIResource(0, 0, 0, 0),
       "Spinner.background", getControl(),
       "Spinner.border", MetalBorders.getTextFieldBorder(),
-      "Spinner.font", new FontUIResource("Dialog", Font.BOLD, 12),
+      "Spinner.font", getControlTextFont(),
       "Spinner.foreground", getControl(),
 
       "SplitPane.background", getControl(),
@@ -1185,20 +1188,26 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "SplitPaneDivider.draggingColor", Color.DARK_GRAY,
 
       "TabbedPane.background", getControlShadow(),
+      "TabbedPane.contentBorderInsets", new InsetsUIResource(2, 2, 3, 3),
+      "TabbedPane.contentOpaque", Boolean.TRUE,
       "TabbedPane.darkShadow", getControlDarkShadow(),
       "TabbedPane.focus", getPrimaryControlDarkShadow(),
-      "TabbedPane.font", new FontUIResource("Dialog", Font.BOLD, 12),
+      "TabbedPane.font", getControlTextFont(),
       "TabbedPane.foreground", getControlTextColor(),
       "TabbedPane.highlight", getControlHighlight(),
       "TabbedPane.light", getControl(),
-      "TabbedPane.selected", getControl(),
+      "TabbedPane.selected", getControl(), // overridden in OceanTheme
       "TabbedPane.selectHighlight", getControlHighlight(),
       "TabbedPane.selectedTabPadInsets", new InsetsUIResource(2, 2, 2, 1),
       "TabbedPane.shadow", getControlShadow(),
-      "TabbedPane.tabAreaBackground", getControl(),
-      "TabbedPane.tabAreaInsets", new InsetsUIResource(4, 2, 0, 6),
+      "TabbedPane.tabAreaBackground", getControl(), // overridden in OceanTheme
+      "TabbedPane.tabAreaInsets", new InsetsUIResource(4, 2, 0, 6), // dito
       "TabbedPane.tabInsets", new InsetsUIResource(0, 9, 1, 9),
       
+      // new properties in OceanTheme:
+      // TabbedPane.contentAreaColor
+      // TabbedPane.unselectedBackground
+      
       "Table.background", getWindowBackground(),
       "Table.focusCellBackground", getWindowBackground(),
       "Table.focusCellForeground", getControlTextColor(),
@@ -1245,7 +1254,8 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "TextPane.selectionBackground", getTextHighlightColor(),
       "TextPane.selectionForeground", getHighlightedTextColor(),
 
-      "TitledBorder.font", new FontUIResource("Dialog", Font.BOLD, 12),
+      "TitledBorder.border", new LineBorderUIResource(getPrimaryControl(), 1),
+      "TitledBorder.font", getControlTextFont(),
       "TitledBorder.titleColor", getSystemTextColor(),
 
       "ToggleButton.background", getControl(),
@@ -1267,12 +1277,14 @@ public class MetalLookAndFeel extends BasicLookAndFeel
       "ToolBar.dockingForeground", getPrimaryControlDarkShadow(),
       "ToolBar.floatingBackground", getMenuBackground(),
       "ToolBar.floatingForeground", getPrimaryControl(),
-      "ToolBar.font", new FontUIResource("Dialog", Font.BOLD, 12),
+      "ToolBar.font", getMenuTextFont(),
       "ToolBar.foreground", getMenuForeground(),
       "ToolBar.highlight", getControlHighlight(),
       "ToolBar.light", getControlHighlight(),
       "ToolBar.shadow", getControlShadow(),
       "ToolBar.border", new MetalBorders.ToolBarBorder(),
+      "ToolBar.rolloverBorder", MetalBorders.getToolbarButtonBorder(),
+      "ToolBar.nonrolloverBorder", MetalBorders.getToolbarButtonBorder(),
 
       "ToolTip.background", getPrimaryControl(),
       "ToolTip.backgroundInactive", getControl(),
@@ -1335,12 +1347,24 @@ public class MetalLookAndFeel extends BasicLookAndFeel
   }
 
   /**
-   * Returns the current theme setting for the Metal L&amp;F.
+   * Returns the current theme for the Metal look and feel.  The default is
+   * an instance of {@link OceanTheme}.
    *
-   * @return the current theme setting for the Metal L&amp;F
+   * @return The current theme (never <code>null</code>).
+   * 
+   * @see #setCurrentTheme(MetalTheme)
    */
   public static MetalTheme getCurrentTheme()
   {
+    if (theme == null)
+      {
+        // swing.metalTheme property documented here:
+        // http://java.sun.com/j2se/1.5.0/docs/guide/swing/1.5/index.html
+        if ("steel".equals(SystemProperties.getProperty("swing.metalTheme")))
+          theme = new DefaultMetalTheme();
+        else
+          theme = new OceanTheme();
+      }
     return theme;
   }
 
