@@ -57,8 +57,7 @@ public class HTML
   /**
    * Represents a HTML attribute.
    */
-  public static class Attribute
-    implements Serializable
+  public static final class Attribute
   {
     /**
      * The action attribute
@@ -293,7 +292,7 @@ public class HTML
     /**
      *  The media attribute
      */
-    public static final Attribute MEDIA = new Attribute("media");
+    static final Attribute MEDIA = new Attribute("media");
 
     /**
      *  The method attribute
@@ -464,49 +463,30 @@ public class HTML
      *  The width attribute
      */
     public static final Attribute WIDTH = new Attribute("width");
+
+    /**
+     * This is used to reflect the pseudo class for the a tag.
+     */
+    static final Attribute PSEUDO_CLASS = new Attribute("_pseudo");
+
+    /**
+     * This is used to reflect the dynamic class for the a tag.
+     */
+    static final Attribute DYNAMIC_CLASS = new Attribute("_dynamic");
+
+    /**
+     * The attribute name.
+     */
     private final String name;
 
     /**
      * Creates the attribute with the given name.
      */
-    protected Attribute(String a_name)
+    private Attribute(String a_name)
     {
       name = a_name;
     }
 
-    /**
-     * Calls compareTo on the tag names (Strings)
-     */
-    public int compareTo(Object other)
-    {
-      return name.compareTo(((Attribute) other).name);
-    }
-
-    /**
-     * The attributes are equal if the names are equal
-     * (ignoring case)
-     */
-    public boolean equals(Object other)
-    {
-      if (other == this)
-        return true;
-
-      if (!(other instanceof Attribute))
-        return false;
-
-      Attribute that = (Attribute) other;
-
-      return that.name.equalsIgnoreCase(name);
-    }
-
-    /**
-     * Returns the hash code which corresponds to the string for this tag.
-     */
-    public int hashCode()
-    {
-      return name == null ? 0 : name.hashCode();
-    }
-
     /**
      * Returns the attribute name. The names of the built-in attributes
      * are always returned in lowercase.
@@ -559,7 +539,6 @@ public class HTML
    * Represents a HTML tag.
    */
   public static class Tag
-    implements Comparable, Serializable
   {
     /**
      * The &lt;a&gt; tag
@@ -789,7 +768,7 @@ public class HTML
     /**
      * The &lt;nobr&gt; tag
      */
-    public static final Tag NOBR = new Tag("nobr");
+    static final Tag NOBR = new Tag("nobr");
 
     /**
      * The &lt;noframes&gt; tag , breaks flow, block tag.
@@ -945,22 +924,22 @@ public class HTML
      * This tag is not included into the array, returned by getAllTags().
      * toString() returns 'comment'. HTML reader synthesizes this tag.
      */
-    public static final Tag COMMENT = new Tag("comment", SYNTETIC);
+    public static final Tag COMMENT = new Tag("comment", SYNTHETIC);
 
     /**
      *  All text content is labeled with this tag.
      *  This tag is not included into the array, returned by getAllTags().
      *  toString() returns 'content'. HTML reader synthesizes this tag.
      */
-    public static final Tag CONTENT = new Tag("content", SYNTETIC);
+    public static final Tag CONTENT = new Tag("content", SYNTHETIC);
 
     /**
      * All text content must be in a paragraph element.
      * If a paragraph didn't exist when content was encountered,
      * a paragraph is manufactured.
-     * toString() returns 'implied'. HTML reader synthesizes this tag.
+     * toString() returns 'p-implied'. HTML reader synthesizes this tag.
      */
-    public static final Tag IMPLIED = new Tag("implied", SYNTETIC);
+    public static final Tag IMPLIED = new Tag("p-implied", SYNTHETIC);
     final String name;
     final int flags;
 
@@ -1046,42 +1025,6 @@ public class HTML
       return (flags & BREAKS) != 0;
     }
 
-    /**
-     * Calls compareTo on the tag names (Strings)
-     */
-    public int compareTo(Object other)
-    {
-      return name.compareTo(((Tag) other).name);
-    }
-
-    /**
-     * The tags are equal if the names are equal (ignoring case).
-     */
-    public boolean equals(Object other)
-    {
-      if (other == this)
-        {
-          return true;
-        }
-
-      if (!(other instanceof Tag))
-        {
-          return false;
-        }
-
-      Tag that = (Tag) other;
-
-      return that.name.equalsIgnoreCase(name);
-    }
-
-    /**
-     * Returns the hash code which corresponds to the string for this tag.
-     */
-    public int hashCode()
-    {
-      return name == null ? 0 : name.hashCode();
-    }
-
     /**
      * Returns the tag name. The names of the built-in tags are always
      * returned in lowercase.
@@ -1144,7 +1087,7 @@ public class HTML
      */
     boolean isSyntetic()
     {
-      return (flags & SYNTETIC) != 0;
+      return (flags & SYNTHETIC) != 0;
     }
 
     private static void unexpected(Exception ex)
@@ -1185,7 +1128,7 @@ public class HTML
   static final int BREAKS = 1;
   static final int BLOCK = 2;
   static final int PREFORMATTED = 4;
-  static final int SYNTETIC = 8;
+  static final int SYNTHETIC = 8;
   private static Map tagMap;
   private static Map attrMap;
 
@@ -1196,6 +1139,7 @@ public class HTML
    */
   public HTML()
   {
+    // Nothing to do here.
   }
 
   /**
