@@ -1,5 +1,5 @@
 /* NO_RESPONSE.java --
-   Copyright (C) 2005 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -46,7 +46,7 @@ import java.io.Serializable;
  *
  * @author Audrius Meskauskas (AudriusA@Bioinformatics.org)
  */
-public class NO_RESPONSE
+public final class NO_RESPONSE
   extends SystemException
   implements Serializable
 {
@@ -58,7 +58,7 @@ public class NO_RESPONSE
   /**
    * Creates a NO_RESPONSE with the default minor code of 0,
    * completion state COMPLETED_NO and the given explaining message.
-   * @param reasom the explaining message.
+   * @param message the explaining message.
    */
   public NO_RESPONSE(String message)
   {
